#!/bin/bash

#
#  Meta install script for STM32-toolchain.
#  written by Gregor Rebel 2011-04-11.
#  
#  This script will automatically run all install_* scripts in current folder.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition
# openSuSE 11.4

Script="$1"


#{ InstallFuncs v0.7 - written by Gregor Rebel

function add2CleanUp() {     # File, Line              adds given line to cleanup file (if not already done)
    CleanUpFile="$1"
    Line="$2"
    
    if [ ! -e "$CleanUpFile" ]; then
      echo "#!/bin/bash"  >$CleanUpFile
      echo ""            >>$CleanUpFile
      chmod +x $CleanUpFile
    fi
    addLine "$CleanUpFile" "$Line"
}
function add2Script() {      # Script,Command,Execute  adds Command to Script; executes Command before if Execute==1  
  ScriptFile=$1
  Command=$2
  Execute=$3
  
  grep "$Command" $ScriptFile >tmp.grep
  AlreadyAdded=`cat tmp.grep`
  if [ "$AlreadyAdded" == "" ]; then
    Error=""
    if [ "$Execute" == "1" ]; then
      $Command || Error="1"
    fi
    if [ "$Error" == "" ]; then
      echo "$ScriptFile: $Command"
      echo "$Command" >>$ScriptFile
    fi
  fi
}
function addLine() {         # File,Line               adds given line to File if not already done
  File2Add2="$1"
  Line="$2"
  
  if [ ! -e $File2Add2 ]; then
    touch $File2Add2
  fi
  echo "grep \"$Line\" $File2Add2" #D
  AlreadyPresent=`grep "$Line" $File2Add2`
  if [ "$AlreadyPresent" == "" ]; then
    echo "$Line" >>$File2Add2
#  else
#    echo "file already up2date: $File2Add2"
  fi
}
function cmd() {             # CMD,[STDOUT]            display + execute given command; records log + errors
  CMD="$1"
  StdOut="$2"
  
  if [ "$StdOut" != "" ]; then
    echo "`pwd`> $CMD >>$StdOut"
    $CMD >>$StdOut
  else
    echo "`pwd`> $CMD"
    $CMD $StdOut 2>CMD.err || CMD_ERROR="$CMD"
    CMDerr=`cat CMD.err`
    if [ "$CMDerr" != "" ]; then
      CMD_ERROR="$CMD"
    fi
  fi
}
function get() {             # URL_Prefix,File,OutFile retrieves file from URL if not already done
  URL_Prefix="$1"
  gFile="$2"
  gOutFile="$3"
  
  if [ "$gOutFile" == "" ]; then
    gOutFile=$gFile
  fi

  if [ ! -e "$gFile" ]; then
    # CMD="wget $1$2"
    # echo "`pwd`> $CMD"
    # $CMD
#pwd
    cmd "wget $URL_Prefix$gFile -O $gOutFile" 
  fi
  if [ ! -e "$gFile" ]; then
    Get_Fails="$Get_Fails $gFile"
  fi
}
function dir() {             # Directory               creates directory if missing
  Directory="$1"
  
  if [ ! -d $Directory ]; then
    mkdir $Directory
  fi
}
function findPrefix() {      #                         loads variable Prefix either with /usr or /usr/local according to system
  if [ -e "/usr/include/math.h" ]; then
    Prefix="/usr"
  else
    Prefix="/usr/local"
  fi
}
function createLink() {      # Source,Link             creates symbolic link (removes old one before)
  Source="$1"
  Link="$2"
  User="$3"
  
  if [ "$User" != "" ]; then
    sudo -u$User rm -v $Link
  else
    rm -v $Link
  fi
  
  if [ "$User" != "" ]; then
    sudo -u$User ln -vs $Source $Link
  else
    ln -vs $Source $Link
  fi
}
function install() {         #                         downloads, compiles + installs software from source tarball
  TestIfInstalled=$1; shift 
  UrlPrefix=$1;       shift
  FileName=$1;        shift
  Suffix=$1;          shift
  UrlSuffix=$1;       shift
  Directory=$1;       shift
  UnpackCommand=$1;   shift
  Configure=$1;       shift
  Make=$1;            shift
  Install=$1;         shift

  # Default-values:  
  # $Suffix          .tar.gz
  # $UrlSuffix       ""
  # $Directory       $FileName
  # $UnpackCommand   "tar xzf", "tar xjf" (for suffix .tar.bz2) or "tar xf" (for suffix .tar) 
  # $Configure       "./configure --prefix='/usr/'"
  # $Make            "make"
  # $Install         "make install"
  
  # Examples:
  # install " " "ftp://ftp.mars.org/pub/mpeg/" "libmad-0.15.1b"
  # install " " "http://downloads.xiph.org/releases/theora/" "libtheora-1.0alpha7" ".tar.bz2"
  # install " " "http://downloads.sourceforge.net/project/inkscape/inkscape/${Version}/" "inkscape-${Version}" ".tar.bz2" "?use_mirror=mesh" ".tar.bz" "inkscape-${Version}"
  # install " " "http://www2.mplayerhq.hu/MPlayer/releases/" "MPlayer-1.0rc1" ".tar.bz2" "MPlayer-1.0rc1" "tar xjf" \
  #             "./configure --cc=gcc410 --host-cc=gcc410 --prefix=/usr --mandir=/usr/share/man --confdir=/etc/mplayer/ --enable-gui --enable-largefiles --enable-menu --enable-mga --enable-xmga --enable-xv --enable-x11 --enable-fbdev --enable-vm  --enable-xvmc  --language=de,en  --enable-mmx  --enable-3dnow  --enable-sse  --enable-sse2  --enable-shm  --with-glib-config=/opt/gnome/bin/glib-config --with-gtk-config=/opt/gnome/bin/gtk-config"

  OriginDirectory=`pwd`
  
  if [ "$ReinstallAll" == "1" ]; then
    TestIfInstalled=""
  fi
  
  if [ $TestIfInstalled ]; then
    echo "skipping already installed '$FileName'"
  else
    if [ "$Suffix" == "" ]; then
      Suffix=".tar.gz"
    fi
    if [ "$Directory" == "" ]; then
      Directory=$FileName
    fi
    if [ "$UnpackCommand" == "" ]; then
      UnpackCommand="tar xzf"
      if [ "$Suffix" == ".tar.bz2" ]; then
        UnpackCommand="tar xjf"
      fi
      if [ "$Suffix" == ".tar.bz" ]; then
        UnpackCommand="tar xjf"
      fi
      if [ "$Suffix" == ".tar" ]; then
        UnpackCommand="tar xf"
      fi
    fi
    if [ "$Configure" == "" ]; then
      Configure="./configure --prefix=/usr"
    fi
    if [ "$Make" == "" ]; then
      Make="make"
    fi
    if [ "$Install" == "" ]; then
      Install="make install"
    fi
    
    echo ""
    echo "installiere $FileName"
    
    if [ ! -e $FileName$Suffix ]; then
      echo "Datei '$FileName$Suffix' nicht gefunden"
      pwd
      echo "hole $UrlPrefix$FileName$Suffix$UrlSuffix"
      wget $UrlPrefix$FileName$Suffix$UrlSuffix
      echo ""
    fi
    
    if [ ! "$UnpackCommand" == " " ]; then
      echo "entpacke mit '$UnpackCommand'"
      $UnpackCommand $FileName$Suffix
      echo "rm -Rf $FileName" >>$CleanUpBatch
    fi
    
    if [ ! -d $Directory ]; then
      echo "Verzeichnis '$Directory' nicht gefunden!"
  pwd
      echo "Verzeichnis '$Directory' nicht gefunden!" >>$OriginDirectory/$FileName.err
    else
      cd $Directory
      LogFileBase="../$FileName"
      if [ "$Directory" == "." ]; then
        LogFileBase="$FileName"
      fi
      Error=""
      echo -n "kompiliere mit $Configure && $Make && $Install || Error='1'."
      $Configure >$OriginDirectory/$FileName.log  2>$OriginDirectory/$FileName.err && echo -n "." && \
      $Make     >>$OriginDirectory/$FileName.log 2>>$OriginDirectory/$FileName.err && echo -n "." && \
      $Install  >>$OriginDirectory/$FileName.log 2>>$OriginDirectory/$FileName.err || \
      Error='1' 
      
      if [ -n "$Error" ]; then
        echo "FEHLER!"
        echo "$FileName" >>$OriginDirectory/$Failed_Installs
        else
        echo "OK"
      fi
    fi
    cd $OriginDirectory
  fi
}
function installPackage() {  # Name1, Name2, ...       installs all named packages via local package manager (zypper or apt-get)
  if [ -x /usr/bin/zypper ]; then
    cmd "sudo zypper install -l -y $@"
  else
    cmd "sudo apt-get -y install $@"
  fi
}
function makeOrig() {        # File                    creates backup file XXX_orig if not present
  bFile="$1"
  
  BakFile="${bFile}_orig"
  if [ ! -e $BakFile ]; then
    cp $bFile $BakFile
  fi
}
function replaceInFile() {   # File,Search,Replace     replaces all occurences of "$Search" in file $File by "$Replace"
    File="$1"
    Search="$2"
    Replace="$3"
    
    echo "replace $File: $Search"
    echo "     -> $File: $Replace"
    #echo "sed -e \"s|$Search|$Replace|g\" $File"
    sed -e "s|$Search|$Replace|g" $File >${File}_2
    mv -f ${File}_2 ${File}
}
function startIfMissing() {  # File,Command,Name       starts Command if File is missing
  
  TestFile=$1
  Command=$2
  Name=$3
  
  if [ ! -e $TestFile ]; then
    echo "Datei nicht gefunden: '$TestFile'"
    if [ ! "$Name" == "" ]; then
      echo "installiere $Name"
    fi
    $Command
  else
    echo "Datei bereits vorhanden: '$TestFile'"
  fi
}
function untgz() {           # Dir,Archiv              unpacks .tar.gz / .tgz archive if Dir is missing
  Dir="$1"
  Archiv="$2"
  

  if [ ! -d $Dir ]; then
    cmd "tar xzf $Archiv"
  fi
}
function untgj() {           # Dir,Archiv              unpacks .tar.bz2 / .tgj archive if Dir is missing
  Dir="$1"
  Archiv="$2"
  

  if [ ! -d $Dir ]; then
    cmd "tar xjf $Archiv"
  fi
}
function unZIP() {           # Dir,Archiv              unpacks .zip archive if Dir is missing
  Dir="$1"
  Archiv="$2"
  

  if [ ! -d $Dir ]; then

    cmd "unzip -o $Archiv" "unzip.log"
  fi
}

#} InstallFuncs v0.7 - written by Gregor Rebel

if [ "$Script" == "" ]; then
  Files=`ls install_*`

  # prepare for installation
  rm 2>/dev/null ErrorOccured AllInstallsOK
  echo "" >cleanup.sh
  for Script in $Files; do
    touch Logs/$Script.log
  done
  
  # createLinks
  echo "#!/bin/bash"              >scripts/createLinks.sh
  echo "# Script created `date`" >>scripts/createLinks.sh
  
  # run all install scripts
  echo "Runing all install scripts."
  echo "You may check progress by inspecting log files via"
  echo "  tail -f `pwd`/Logs/*"
  echo ""
  for Script in $Files; do
    if [ ! -e ErrorOccured ]; then
      Error=""
      $0 $Script || Error="1"
      if [ "$Error" != "" ]; then
        echo "ERROR occured during $Script !"
        exit 6
      fi
    fi
  done
  
  # remove temporary files
  chmod +x cleanup.sh
  ./cleanup.sh
  
  # installation finished successfully
  touch AllInstallsOK
  echo ""
  echo "All tools have been installed successfully."
  echo "Note: Your .bashrc has been updated."
  echo "      Start a new shell to use the new environment!"
  echo ""
else
  echo -n "running `pwd`/$Script.."
  dir Logs
  echo "$Script started @`date`" >Logs/$Script.log
  ./$Script >Logs/$Script.log 2>Logs/$Script.err && OK="1"
  if [ "$OK" == "1" ]; then
    echo "OK"
  else
    echo "ERROR"
    echo ""
    echo "Check logfiles:"
    find `pwd`/Logs/ -name $Script.* -exec echo "less {}" \;
    touch ErrorOccured
    exit 5
  fi
fi
